function [sys,x0,str,ts,simStateCompliance] = distill(t,y,u,flag)

switch flag,
    
    %%%%%%%%%%%%%%%%%%
    % Initialization %
    %%%%%%%%%%%%%%%%%%
    case 0,
        [sys,x0,str,ts,simStateCompliance]=mdlInitializeSizes();
        
        %%%%%%%%%%%%%%%
        % Derivatives %
        %%%%%%%%%%%%%%%
    case 1,
        sys=mdlDerivatives(t,y,u);
        
        %%%%%%%%%%
        % Update %
        %%%%%%%%%%
    case 2,
        sys=mdlUpdate(t,y,u);
        
        %%%%%%%%%%%
        % Outputs %
        %%%%%%%%%%%
    case 3,
        sys=mdlOutputs(t,y,u);
        
        %%%%%%%%%%%%%%%%%%%%%%%
        % GetTimeOfNextVarHit %
        %%%%%%%%%%%%%%%%%%%%%%%
    case 4,
        sys=mdlGetTimeOfNextVarHit(t,y,u);
        
        %%%%%%%%%%%%%
        % Terminate %
        %%%%%%%%%%%%%
    case 9,
        sys=mdlTerminate(t,y,u);
        
        %%%%%%%%%%%%%%%%%%%%
        % Unexpected flags %
        %%%%%%%%%%%%%%%%%%%%
    otherwise
        DAStudio.error('Simulink:blocks:unhandledFlag', num2str(flag));
        
end

% end sfuntmpl

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts,simStateCompliance]=mdlInitializeSizes()

%
% call simsizes for a sizes structure, fill it in and convert it to a
% sizes array.
%
% Note that in this example, the values are hard coded.  This is not a
% recommended practice as the characteristics of the block are typically
% defined by the S-function parameters.
%
sizes = simsizes;

sizes.NumContStates  = 20;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 5;
sizes.NumInputs      = 4;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;   % at least one sample time is needed

sys = simsizes(sizes);
%
% initialize the initial conditions
x0 = ones(20,1)*298.0;

%
% str is always an empty matrix
%
str = [];

%
% initialize the array of sample times
%
ts  = [0 0];

% Specify the block simStateCompliance. The allowed values are:
%    'UnknownSimState', < The default setting; warn and assume DefaultSimState
%    'DefaultSimState', < Same sim state as a built-in block
%    'HasNoSimState',   < No sim state
%    'DisallowSimState' < Error out when saving or restoring the model sim state
simStateCompliance = 'UnknownSimState';

% end mdlInitializeSizes

%==========================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%=============================================================================
%
function sys=mdlDerivatives(t,x,u)
%
% 1-D heat conduction, unsteady state

n = size(x,1);

% Inputs (3):
% Left Surface Temperature (K)
Temp(1) = u(1);

% Temperature of surroundings
T_amb = u(2); % 298.15;

% Overall heat transfer coefficient
h = u(3); % 2;

% Type of material (0 = silver, 1 = aluminum)
type = u(4);
material = round(type);

% States (n):
% Temperatures at points along the rod
Temp(2:n+1) = x(1:n,1)';

% Parameters:
% Radius of the rod
radius = 0.03; % 3 cm
% Length of the rod
length = 0.10;  % 30 cm

% Thermal conductivity
k_silver = [436.13 -0.011922 -0.000084465 3.3463E-08 0];
k_alum = [165.84 0.49305 -0.0011114 9.8024E-07 -3.2368E-10];
if material == 0;
    A = k_silver;
else
    A = k_alum;
end
for i = 1:n+1,
    k(i) = (A(1) + A(2)*Temp(i) + A(3)*Temp(i)^2 + A(4)*Temp(i)^3 + A(5)*Temp(i)^4);
end
% use the harmonic mean to estimate the thermal conductivity at the control volume borders
for i = 1:n
    k_hm(i) = 2*k(i+1)*k(i) / (k(i)+k(i+1));
end

% Solid density
rho_silver = [102.79 -0.0186 0 0 0];
rho_alum = [102.83 -0.00873 0 0 0];
if material == 0;
    A = rho_silver;
else
    A = rho_alum;
end
for i = 1:n+1,
    rho(i) = A(1) + A(2)*Temp(i) + A(3)*Temp(i)^2 + A(4)*Temp(i)^3 + A(5)*Temp(i)^4;
end

% Solid heat capacity
cp_silver = [24710 1.14 0.00388 0 0];
cp_alum = [22149 5.7062 0.0067408 0 0];
if material == 0;
    A = cp_silver;
else
    A = cp_alum;
end
for i = 1:n+1,
    cp(i) = A(1) + A(2)*Temp(i) + A(3)*Temp(i)^2 + A(4)*Temp(i)^3 + A(5)*Temp(i)^4;
end

% Determine equal spacing between nodes
dx = length / n;
% Surface area for each section
As = 2 * pi * radius * dx;
% Surface area for the end or cross-sectional areas
Ae = pi * radius^2;

% Compute xdot:
for i = 2:n,
    xdot(i-1,1) = (1/(rho(i)*cp(i)*Ae*dx)) ...
        * (k_hm(i)*Ae*(Temp(i+1)-Temp(i))/dx ...
        - k_hm(i-1)*Ae*(Temp(i)-Temp(i-1))/dx ...
        - h * As * (Temp(i)-T_amb));
end
xdot(n,1) = (1/(rho(n+1)*cp(n+1)*Ae*dx)) ...
    * (-k_hm(n)*Ae*(Temp(n+1)-Temp(n))/dx ...
    - h * (As+Ae) * (Temp(n+1)-T_amb));

% derivatives
sys = xdot;
% end mdlDerivatives

%
%=============================================================================
% mdlUpdate
% Handle discrete state updates, sample time hits, and major time step
% requirements.
%=============================================================================
%
function sys=mdlUpdate(t,y,u)

sys = [];

% end mdlUpdate

%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,y,u)

sys = [y(1) y(5) y(10) y(15) y(20)];

% end mdlOutputs

%
%=============================================================================
% mdlGetTimeOfNextVarHit
% Return the time of the next hit for this block.  Note that the result is
% absolute time.  Note that this function is only used when you specify a
% variable discrete-time sample time [-2 0] in the sample time array in
% mdlInitializeSizes.
%=============================================================================
%
function sys=mdlGetTimeOfNextVarHit(t,y,u)

sampleTime = 1;    %  Example, set the next hit to be one second later.
sys = t + sampleTime;

% end mdlGetTimeOfNextVarHit

%
%=============================================================================
% mdlTerminate
% Perform any end of simulation tasks.
%=============================================================================
%
function sys=mdlTerminate(t,y,u)

sys = [];

% end mdlTerminate
